const crypto = require("crypto");
const { promisify } = require("util");
const jwt = require("jsonwebtoken");
const User = require("./../models/userModel");
const catchAsync = require("./../utils/catchAsync");
const AppError = require("./../utils/appError");
const Email = require("./../utils/email");
const upload = require("../utils/multerConfig");
const bcrypt = require("bcryptjs");
const TempUser = require("../models/tempUserModel"); // Temporary user model
const { body, validationResult } = require("express-validator");
const {
  generateOTP,
  sendOTPEmailForVerification,
} = require("../utils/otpService");
const { isFloatLocales } = require("validator");

const signToken = (id) => {
  return jwt.sign({ id }, process.env.JWT_SECRET, {
    expiresIn: process.env.JWT_EXPIRES_IN,
  });
};

const createSendToken = (user, statusCode, req, res) => {
  const token = signToken(user._id);

  // Ensure the expiration time is a valid number
  const cookieExpireInDays =
    parseInt(process.env.JWT_COOKIE_EXPIRE_IN, 10) || 90; // Default to 90 days if not set

  // Define cookie options
  const cookieOptions = {
    expires: new Date(
      Date.now() + cookieExpireInDays * 24 * 60 * 60 * 1000 // Convert days to milliseconds
    ),
    httpOnly: true, // Prevent client-side script access
    secure: req.secure || req.headers["x-forwarded-proto"] === "https", // Ensure secure cookies in production
    sameSite: "strict", // Helps mitigate CSRF attacks
  };

  // Send cookie
  res.cookie("jwt", token, cookieOptions);

  // Remove sensitive data from the user object
  user.password = undefined;

  // Send response
  res.status(statusCode).json({
    status: true,
    token,
    data: {
      user,
    },
  });
};

exports.signupName = [
  // Validate firstName and lastName
  body("firstName")
    .trim()
    .notEmpty()
    .withMessage("Please enter your first name.")
    .isAlpha()
    .withMessage("First name can only contain alphabets."),

  body("lastName")
    .trim()
    .notEmpty()
    .withMessage("Please enter your last name.")
    .isAlpha()
    .withMessage("Last name can only contain alphabets."),

  catchAsync(async (req, res, next) => {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      return res.status(400).json({
        status: false,
        error: errors.array()[0].msg,
      });
    }

    const { firstName, lastName } = req.body;

    try {
      // Generate a random email string
      const randomEmail =
        crypto.randomBytes(6).toString("hex") + "@example.com";

      // Create the TempUser with the random email
      const tempUser = await TempUser.create({
        firstName,
        lastName,
        email: randomEmail, // Set random email
      });

      res.status(200).json({
        status: true,
        message: "Proceed to the next step: Email",
        tempUserId: tempUser._id,
      });
    } catch (error) {
      console.error(error);
      res.status(500).json({
        status: false,
        error: "Something went wrong while processing name.",
      });
    }
  }),
];

exports.signupEmail = [
  body("email")
    .isEmail()
    .withMessage("Please provide a valid email address.")
    .normalizeEmail(),

  catchAsync(async (req, res, next) => {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      return res.status(400).json({
        status: false,
        error: errors.array()[0].msg,
      });
    }

    const { email } = req.body;
    const { tempUserId } = req.params;

    try {
      // Check if the TempUser exists
      const tempUser = await TempUser.findById(tempUserId);
      if (!tempUser) {
        return res.status(404).json({
          status: false,
          error: "No user found with this ID.",
        });
      }

      // Check if the email is already registered
      const user = await User.findOne({ email });
      if (user) {
        return res.status(400).json({
          status: false,
          error: "Email is already registered.",
        });
      }

      // Update the TempUser with the provided email
      await TempUser.findByIdAndUpdate(tempUserId, { email });

      res.status(200).json({
        status: true,
        message: "Proceed to the next step: Password",
      });
    } catch (error) {
      console.error(error);
      res.status(500).json({
        status: false,
        error: "Something went wrong while processing email.",
      });
    }
  }),
];

exports.signupPassword = [
  body("password")
    .notEmpty()
    .withMessage("Please enter a password.")
    .isLength({ min: 8 })
    .withMessage("Password must be at least 8 characters long.")
    .matches(/[0-9]/)
    .withMessage("Password must contain at least one number.")
    .matches(/[A-Z]/)
    .withMessage("Password must contain at least one uppercase letter.")
    .matches(/[a-z]/)
    .withMessage("Password must contain at least one lowercase letter."),

  catchAsync(async (req, res, next) => {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      return res.status(400).json({
        status: false,
        error: errors.array()[0].msg,
      });
    }

    const { password } = req.body;
    const { tempUserId } = req.params;
    // console.log(password);

    try {
      // Check if the TempUser exists
      const tempUser = await TempUser.findById(tempUserId);
      if (!tempUser) {
        return res.status(404).json({
          status: false,
          error: "No user found with this ID.",
        });
      }

      // Hash the password before saving it
      // const hashedPassword = await bcrypt.hash(password, 12);

      // Update the TempUser with the hashed password
      await TempUser.findByIdAndUpdate(tempUserId, {
        password: password,
      });

      res.status(200).json({
        status: true,
        message:
          "Password saved successfully. Proceed to the next step: Profile Info",
      });
    } catch (error) {
      console.error(error);
      res.status(500).json({
        status: false,
        message: "Something went wrong while processing password.",
      });
    }
  }),
];

exports.signupProfileInfo = catchAsync(async (req, res, next) => {
  const { location, recentJob, company } = req.body;
  const { tempUserId } = req.params;

  try {
    // Check if all required fields are provided
    if (!recentJob || !company) {
      return res.status(400).json({
        status: false,
        error: "Please provide all required fields.",
      });
    }

    // Check if the TempUser exists
    const tempUser = await TempUser.findById(tempUserId);
    if (!tempUser) {
      return res.status(404).json({
        status: false,
        error: "No user found with this ID.",
      });
    }

    // Update TempUser profile information
    await TempUser.findByIdAndUpdate(tempUserId, {
      location,
      recentJob,
      company,
    });

    res.status(200).json({
      status: true,
      message:
        "Profile info saved successfully. Proceed to the next step: Interests",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({
      status: false,
      error: "Something went wrong while processing profile info.",
    });
  }
});
exports.signupInterests = catchAsync(async (req, res, next) => {
  const { interests } = req.body;
  const { tempUserId } = req.params;

  try {
    // Check if at least 5 interests are provided
    if (!interests || interests.length < 5) {
      return res.status(400).json({
        status: false,
        error: "Please select at least 5 interests.",
      });
    }

    // Check if the TempUser exists
    const tempUser = await TempUser.findById(tempUserId);
    if (!tempUser) {
      return res.status(404).json({
        status: false,
        error: "No user found with this ID.",
      });
    }

    // Update TempUser interests
    await TempUser.findByIdAndUpdate(tempUserId, { interests });

    res.status(200).json({
      status: true,
      message:
        "Interests saved successfully. Proceed to the next step: Upload Photo",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({
      status: false,
      error: "Something went wrong while saving interests.",
    });
  }
});

exports.signupPhoto = [
  upload.single("photo"),
  async (req, res, next) => {
    const { tempUserId } = req.params;

    try {
      // Check if the TempUser exists
      const tempUser = await TempUser.findById(tempUserId);
      if (!tempUser) {
        return res.status(404).json({
          status: false,
          error: "No user found with this ID.",
        });
      }

      // Upload photo if a file is provided
      if (req.file) {
        await TempUser.findByIdAndUpdate(tempUserId, { photo: req.file.path });
      }

      // Generate OTP
      const otp = generateOTP();

      // Push the OTP to the otps array
      tempUser.otps.push({
        otp: otp,
        otpExpires: Date.now() + 10 * 60 * 1000, // OTP expiry time
      });

      // Save the updated tempUser document
      await tempUser.save();

      // Optionally, send OTP email
      await sendOTPEmailForVerification(tempUser.email, otp);

      res.status(200).json({
        status: true,
        message: "Photo uploaded and OTP sent.",
        otp,
      });
    } catch (error) {
      console.error(error);
      res.status(500).json({
        status: false,
        error: "Something went wrong during the process.",
      });
    }
  },
];

exports.resendOTP = async (req, res) => {
  try {
    const { tempUserId } = req.params;

    // Fetch the TempUser from the database
    const tempUser = await TempUser.findById(tempUserId);

    if (!tempUser) {
      return res.status(404).json({ status: false, error: "User not found." });
    }

    // Generate a new OTP
    const otp = generateOTP();

    // Update OTP in the TempUser's otps array
    tempUser.otps.push({
      otp: otp,
      otpExpires: Date.now() + 10 * 60 * 1000, // OTP valid for 10 minutes
    });

    // Save the updated TempUser document
    await tempUser.save();

    // Optionally, resend OTP via email
    await sendOTPEmailForVerification(tempUser.email, otp);

    // Return success message
    res.status(200).json({
      status: true,
      message: "A new OTP has been sent to your email.",
    });
  } catch (error) {
    console.error("Error while resending OTP:", error);
    res.status(500).json({
      status: false,
      error: "Something went wrong while resending the OTP.",
    });
  }
};

exports.verifyOTP = async (req, res, next) => {
  const { otp } = req.body;
  const { tempUserId } = req.params;

  try {
    // Fetch the TempUser from the database
    const tempUser = await TempUser.findById(tempUserId);

    if (!tempUser) {
      return res.status(404).json({ status: false, error: "User not found." });
    }

    // Check for a valid OTP in the otps array
    const validOtp = tempUser.otps.find(
      (otpEntry) =>
        otpEntry.otp === otp &&
        Date.now() < new Date(otpEntry.otpExpires).getTime()
    );

    if (!validOtp) {
      return res.status(400).json({
        status: false,
        error: "Invalid or expired OTP.",
      });
    }

    // // Debugging the password field (this can be removed after checking)
    // console.log("Password field value: ", tempUser.password);

    // Check if password is set for the user
    if (!tempUser.password) {
      return res.status(400).json({
        status: false,
        error: "Password is missing in the user data.",
      });
    }

    // If OTP is valid, create the final user in the User collection
    const user = await User.create({
      firstName: tempUser.firstName,
      lastName: tempUser.lastName,
      email: tempUser.email,
      password: tempUser.password,
      location: tempUser.location,
      recentJob: tempUser.recentJob,
      company: tempUser.company,
      interests: tempUser.interests,
      photo: tempUser.photo || null,
    });

    // Delete the temporary user document after successful verification
    await TempUser.findByIdAndDelete(tempUserId);

    // Send the created user a JWT token
    createSendToken(user, 201, req, res);
  } catch (error) {
    console.error("Error while verifying OTP:", error);
    res.status(500).json({
      status: false,
      error: "Error occurred while verifying OTP.",
    });
  }
};

exports.login = catchAsync(async (req, res, next) => {
  const { email, password } = req.body;

  // 1) Check if email and password exist
  if (!email || !password) {
    return res.status(400).json({
      status: false,
      Error: "Please provide email and password!",
    });
  }

  // 2) Check if user exists && password is correct
  const user = await User.findOne({ email }).select("+password");

  if (!user || !(await user.correctPassword(password, user.password))) {
    return res.status(401).json({
      status: false,
      Error: "Incorrect email or password",
    });
  }

  // 3) If everything is ok, send token to client
  createSendToken(user, 200, req, res);
});

// Protect middleware
exports.protect = async (req, res, next) => {
  try {
    // 1) Get the token from the Authorization header
    let token;
    if (
      req.headers.authorization &&
      req.headers.authorization.startsWith("Bearer")
    ) {
      token = req.headers.authorization.split(" ")[1]; // Get the token part from the "Bearer <token>" string
    }

    // 2) If no token, send an unauthorized response
    if (!token) {
      return res.status(401).json({
        status: false,
        message: "You are not logged in! Please log in to get access.",
      });
    }

    // 3) Verify the token
    const decoded = jwt.verify(token, process.env.JWT_SECRET); // process.env.JWT_SECRET should match your secret key

    // 4) Check if user still exists
    const currentUser = await User.findById(decoded.id); // decoded.id should contain the user ID from the token
    if (!currentUser) {
      return res.status(401).json({
        status: false,
        message: "The user belonging to this token no longer exists.",
      });
    }

    // 5) Grant access to the protected route
    req.user = currentUser; // Store the user object in the request to use in the controller
    // console.log(req.user);
    next(); // Move to the next middleware or controller
  } catch (error) {
    console.error(error);
    return res.status(401).json({
      status: false,
      message: "Invalid token or you are not authorized.",
    });
  }
};
// Only for rendered pages, no errors!
exports.isLoggedIn = async (req, res, next) => {
  if (req.cookies.jwt) {
    try {
      // 1) verify token
      const decoded = await promisify(jwt.verify)(
        req.cookies.jwt,
        process.env.JWT_SECRET
      );

      // 2) Check if user still exists
      const currentUser = await User.findById(decoded.id);
      if (!currentUser) {
        return next();
      }

      // 3) Check if user changed password after the token was issued
      if (currentUser.changedPasswordAfter(decoded.iat)) {
        return next();
      }

      // THERE IS A LOGGED IN USER
      res.locals.user = currentUser;
      return next();
    } catch (err) {
      return next();
    }
  }
  next();
};

exports.restrictTo = (...roles) => {
  return (req, res, next) => {
    // roles ['admin', 'lead-guide']. role='user'
    if (!roles.includes(req.user.role)) {
      return next(
        new AppError("You do not have permission to perform this action", 403)
      );
    }

    next();
  };
};

exports.updatePassword = [
  // Validate current password, new password, and confirm password
  body("passwordCurrent")
    .notEmpty()
    .withMessage("Please enter your current password."),

  body("password")
    .notEmpty()
    .withMessage("Please enter a new password.")
    .isLength({ min: 8 })
    .withMessage("Password must be at least 8 characters long.")
    .matches(/[0-9]/)
    .withMessage("Password must contain at least one number.")
    .matches(/[A-Z]/)
    .withMessage("Password must contain at least one uppercase letter.")
    .matches(/[a-z]/)
    .withMessage("Password must contain at least one lowercase letter.")
    .trim(), // Trim whitespace around the password

  body("passwordConfirm")
    .notEmpty()
    .withMessage("Please confirm your new password.")
    .custom((value, { req }) => value === req.body.password)
    .withMessage("Passwords do not match."),

  // Actual password update logic
  catchAsync(async (req, res, next) => {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      return res.status(400).json({
        status: false,
        errors: errors.array()[0].msg, // Return only the first validation error
      });
    }

    // 1) Get user from the database, including the password field
    const user = await User.findById(req.user.id).select("+password");
    if (!user) {
      return res.status(404).json({
        status: false,
        error: "User not found.",
      });
    }

    // 2) Check if the provided current password is correct
    const isPasswordCorrect = await user.correctPassword(
      req.body.passwordCurrent,
      user.password
    );
    if (!isPasswordCorrect) {
      return res.status(401).json({
        status: false,
        error: "Your current password is incorrect.",
      });
    }

    // 3) If the current password is correct, update the password
    user.password = req.body.password;
    user.passwordConfirm = req.body.passwordConfirm;
    await user.save();

    // 4) Log the user in and send JWT token
    createSendToken(user, 200, req, res); // Function to create and send the JWT
  }),
];
exports.logout = catchAsync(async (req, res, next) => {
  try {
    // Log the user out
    req.logout((err) => {
      if (err) {
        return next(new AppError("Error logging out", 500)); // Handle logout errors
      }

      // Clear the session
      req.session.destroy((sessionErr) => {
        if (sessionErr) {
          return next(new AppError("Error clearing session", 500)); // Handle session clear errors
        }

        // Set the JWT cookie to 'loggedout' with a short expiry
        res.cookie("jwt", "loggedout", {
          expires: new Date(Date.now() + 10 * 1000), // Cookie expires in 10 seconds
          httpOnly: true, // Ensures cookie is not accessible via JavaScript
          secure: process.env.NODE_ENV === "production", // Ensure the cookie is only sent over HTTPS in production
          sameSite: "strict", // Ensures the cookie is sent only for same-site requests
        });

        // Send success response
        res.status(200).json({
          status: true,
          message: "Logged out successfully",
        });
      });
    });
  } catch (err) {
    return next(new AppError("Something went wrong during logout", 500));
  }
});
