const User = require("../models/userModel");
const upload = require("../utils/multerConfig"); // Import the multer config
const mongoose = require("mongoose");
const path = require("path");
const moment = require("moment");

// Define the route to get the current logged-in user's profile
exports.getCurrentUserProfile = async (req, res) => {
  try {
    // The user object should be available in req.user after authentication
    const user = req.user;

    // Check if the user object is available (if user is logged in)
    if (!user) {
      return res.status(401).json({
        status: false,
        error: "You must be logged in to view this information.",
      });
    }

    // Return the current user's profile (you can choose which fields to send)
    return res.status(200).json({
      status: true,
      data: {
        user,
      },
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: false,
      error: "An error occurred while fetching the user profile.",
    });
  }
};
// Controller for getting all users
exports.getAllUsers = async (req, res) => {
  try {
    const queryObj = { ...req.query }; // Copy query parameters
    const excludedFields = ["page", "sort", "limit", "fields"];
    excludedFields.forEach((el) => delete queryObj[el]);

    // Advanced filtering
    let queryStr = JSON.stringify(queryObj);
    queryStr = queryStr.replace(/\b(gte|gt|lte|lt)\b/g, (match) => `$${match}`);

    let query = User.find(JSON.parse(queryStr));

    // Sorting
    if (req.query.sort) {
      const sortBy = req.query.sort.split(",").join(" ");
      query = query.sort(sortBy);
    } else {
      query = query.sort("-createdAt"); // Default sorting
    }

    // Field limiting
    if (req.query.fields) {
      const fields = req.query.fields.split(",").join(" ");
      query = query.select(fields);
    } else {
      query = query.select("-__v"); // Exclude __v by default
    }

    // Pagination
    const page = req.query.page * 1 || 1;
    const limit = req.query.limit * 1 || 10;
    const skip = (page - 1) * limit;
    query = query.skip(skip).limit(limit);

    if (req.query.page) {
      const numUsers = await User.countDocuments();
      if (skip >= numUsers) throw new Error("This page does not exist");
    }

    // Execute query
    const users = await query;

    // Send response
    res.status(200).json({
      status: true,
      results: users.length,
      data: {
        users,
      },
    });
  } catch (err) {
    console.error(err);
    res.status(500).json({
      status: false,
      Error: "Something went wrong while fetching users",
    });
  }
};
// Get user by ID
exports.getUserById = async (req, res) => {
  try {
    // Get the user ID from request params
    const { id } = req.params;
    // console.log(id);

    // Validate the ID format
    if (!mongoose.Types.ObjectId.isValid(id)) {
      return res.status(400).json({
        status: false,
        error: "Invalid User ID format",
      });
    }

    // Find the user by ID
    const user = await User.findById(id);

    // Check if user exists
    if (!user) {
      return res.status(404).json({
        status: false,
        error: "User not found",
      });
    }

    // Send response
    res.status(200).json({
      status: true,
      data: {
        user,
      },
    });
  } catch (err) {
    console.error(err);
    res.status(500).json({
      status: false,
      error: "An error occurred while fetching the user",
    });
  }
};

//Controller for the get Suggested users
exports.getSuggestedUsers = async (req, res) => {
  try {
    // Debug: Log the current user object to verify structure
    // console.log("Logged-in user data:", req.user);

    // Get the interests of the currently logged-in user
    const { interests, _id: currentUserId } = req.user;

    // Debug: Log the currentUserId to verify its format
    // console.log("Current User ID:", currentUserId);

    // Validate the currentUserId to ensure it's a valid MongoDB ObjectId
    // Check directly without converting it to a string
    if (!mongoose.Types.ObjectId.isValid(currentUserId)) {
      return res.status(400).json({
        status: false,
        error: "Invalid User ID format",
      });
    }

    // Validate that the interests are an array and have values
    if (!interests || !Array.isArray(interests) || interests.length === 0) {
      return res.status(400).json({
        status: false,
        error: "You have no interests defined in your profile.",
      });
    }

    // Debugging: Log the interests of the current user
    console.log("User interests:", interests);

    // Fetch users with matching interests, excluding the current user
    const users = await User.find({
      _id: { $ne: currentUserId }, // Exclude the current user
      interests: { $in: interests }, // Match at least one interest
    });

    // Check if any users are found
    if (!users || users.length === 0) {
      return res.status(404).json({
        status: false,
        error: "No users found with matching interests.",
      });
    }

    // Debugging: Log the users found with matching interests
    console.log("Users with matching interests:", users);

    // Sort users by the number of matching interests
    const sortedUsers = users
      .map((user) => {
        const matchedInterests = user.interests.filter((interest) =>
          interests.includes(interest)
        );
        return { ...user._doc, matchedCount: matchedInterests.length };
      })
      .sort((a, b) => b.matchedCount - a.matchedCount); // Sort by highest matches

    // Send response
    res.status(200).json({
      status: true,
      results: sortedUsers.length,
      data: {
        users: sortedUsers,
      },
    });
  } catch (err) {
    console.error(err);
    res.status(500).json({
      status: false,
      error: "An error occurred while fetching suggested users.",
    });
  }
};

// Controller for updating profile picture
exports.updateProfilePicture = (req, res) => {
  // Handle the upload process
  upload.single("photo")(req, res, async (err) => {
    try {
      // Handle any errors that might occur during file upload
      if (err) {
        return res.status(400).json({
          status: false,
          error: err.message || "An error occurred while uploading the photo.",
        });
      }

      // Check if the user exists based on user ID (assumed to be set in req.user)
      const user = await User.findById(req.user.id);
      if (!user) {
        return res.status(404).json({ status: false, error: "User not found" });
      }

      // If no file is uploaded, return an error
      if (!req.file) {
        return res.status(400).json({
          status: false,
          error: "No photo uploaded. Please select a photo.",
        });
      }

      // Get the uploaded file's path
      let photoPath = req.file.path;

      // Normalize the path (replace backslashes with forward slashes)
      photoPath = photoPath.replace(/\\/g, "/");

      // // URL encode the photo path (handle spaces and special characters)
      // photoPath = encodeURIComponent(photoPath);

      // Update the user's profile with the new photo path
      user.photo = `${photoPath}`; // Assuming your photos are stored in /uploads/photos

      // Save the updated user data to the database
      await user.save();

      // Respond with the updated user data
      return res.status(200).json({
        status: true,
        message: "Profile picture updated successfully!",
        data: {
          photo: user.photo, // Return the updated photo file path
        },
      });
    } catch (error) {
      console.error(error);
      return res.status(500).json({
        status: false,
        error: "An error occurred while updating the profile picture.",
      });
    }
  });
};

// Controller to update bio and social links
exports.updateProfileBioLink = async (req, res) => {
  try {
    // Check if the request body is empty
    if (!req.body || Object.keys(req.body).length === 0) {
      return res.status(400).json({
        status: false,
        error: "Please provide bio and/or social links to update.",
      });
    }

    const { bio, socialLinks } = req.body;

    // Check if the user exists (assuming req.user.id is populated from authentication middleware)
    const user = await User.findById(req.user.id);
    if (!user) {
      return res.status(404).json({
        status: false,
        error: "User not found.",
      });
    }

    // Update bio if provided
    if (bio !== undefined) {
      user.bio = bio;
    }

    // Update social links if provided
    if (socialLinks) {
      if (socialLinks.linkedin !== undefined) {
        user.socialLinks.linkedin = socialLinks.linkedin || ""; // Set to empty string if blank
      }
      if (socialLinks.github !== undefined) {
        user.socialLinks.github = socialLinks.github || "";
      }
      if (socialLinks.twitter !== undefined) {
        user.socialLinks.twitter = socialLinks.twitter || "";
      }
      if (socialLinks.facebook !== undefined) {
        user.socialLinks.facebook = socialLinks.facebook || "";
      }
      if (socialLinks.website !== undefined) {
        user.socialLinks.website = socialLinks.website || "";
      }
      if (socialLinks.whatsapp !== undefined) {
        user.socialLinks.whatsapp = socialLinks.whatsapp || "";
      }
      if (socialLinks.instagram !== undefined) {
        user.socialLinks.instagram = socialLinks.instagram || "";
      }
    }

    // Save the updated user document
    await user.save();

    // Respond with the updated user data
    return res.status(200).json({
      status: true,
      message: "Profile updated successfully!",
      data: {
        bio: user.bio,
        socialLinks: user.socialLinks,
      },
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: false,
      message: "An error occurred while updating the profile.",
    });
  }
};
// Controller to update the current user's profile
exports.updateCurrentUserProfile = async (req, res) => {
  try {
    // Destructure the incoming request body
    const {
      firstname,
      lastname,
      othername,
      recentJob,
      company,
      email,
      phone,
      country,
    } = req.body;

    // Validate the required fields
    if (!firstname || !lastname || !email || !recentJob) {
      return res.status(400).json({
        status: false,
        error: "Firstname, lastname, recentJob, and email are required.",
      });
    }

    // Validate email format
    const emailRegex = /^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/;
    if (!emailRegex.test(email)) {
      return res.status(400).json({
        status: false,
        error: "Please provide a valid email address.",
      });
    }

    // Check if the email is already in use by another user
    const existingUser = await User.findOne({ email });
    if (
      existingUser &&
      existingUser._id.toString() !== req.user._id.toString()
    ) {
      return res.status(400).json({
        status: false,
        error: "Email already exists.",
      });
    }

    // Proceed if no errors and if the user is authenticated (this is set by the 'protect' middleware)
    const user = req.user;

    // Update user fields
    user.firstname = firstname;
    user.lastname = lastname;
    user.othername = othername || user.othername; // Keep old value if not provided
    user.recentJob = recentJob;
    user.company = company || user.company; // Keep old value if not provided
    user.email = email;
    user.phone = phone || user.phone; // Keep old value if not provided
    user.country = country || user.country; // Keep old value if not provided

    // If a new photo is uploaded, update the photo path
    if (req.file) {
      const photoPath = path.join("uploads", "photos", req.file.filename);
      // Encode the photo path to be URL safe
      user.photo = `/${photoPath.replace(/\\/g, "/")}`; // Replace backslashes with forward slashes
    }

    // Save the updated user information
    await user.save();

    // Return success response with the correctly formatted photo path
    return res.status(200).json({
      status: true,
      message: "User profile updated successfully.",
      data: user,
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: false,
      error: "An error occurred while updating the user profile.",
    });
  }
};
// Controller to get the current user's profile completion percentage
exports.getProfileCompletion = async (req, res) => {
  try {
    // `req.user` is set by the `protect` middleware
    const user = req.user;

    // If no user found in request, return an error (this is a fallback, ideally won't happen)
    if (!user) {
      return res.status(404).json({
        status: false,
        message: "User not found",
      });
    }

    // Calculate the profile completion percentage
    const profileCompletion = user.calculateProfileCompletion();

    return res.status(200).json({
      status: true,
      message: "Profile completion fetched successfully",
      data: {
        profileCompletion,
      },
    });
  } catch (error) {
    console.error(error);
    return res.status(500).json({
      status: false,
      message: "An error occurred while calculating the profile completion.",
    });
  }
};
// Route to toggle the user's visibility
exports.toggleVisibility = async (req, res) => {
  try {
    // Assuming you have some kind of authentication to get the current logged-in user
    const user = req.user; // This should be set by some authentication middleware

    // Toggle the 'visible' field
    user.visible = !user.visible;

    // Save the updated user to the database
    await user.save();

    // Send response
    res.status(200).json({
      status: true,
      message: `Profile visibility is now ${
        user.visible ? "visible" : "hidden"
      }`,
    });
  } catch (err) {
    console.error(err);
    res.status(500).json({
      status: false,
      message: "An error occurred while updating visibility",
    });
  }
};

// Controller to set profile visibility
exports.setProfileVisibility = async (req, res) => {
  try {
    const { lockForDuration, unit } = req.body; // lockForDuration (e.g., 1, 2, etc.) and unit (e.g., 'minutes', 'hours', 'days')

    // Validation: Check if lockForDuration and unit are provided
    if (!lockForDuration || !unit) {
      return res.status(400).json({
        status: false,
        error: "lockForDuration and unit are required.",
      });
    }

    // Check if unit is valid
    const validUnits = ["minutes", "hours", "days"];
    if (!validUnits.includes(unit)) {
      return res.status(400).json({
        status: false,
        error: 'Invalid time unit. Please use "minutes", "hours", or "days".',
      });
    }

    // Convert the duration to milliseconds using moment.js
    const durationInMillis = moment
      .duration(lockForDuration, unit)
      .asMilliseconds();
    const visibilityEndTime = new Date(Date.now() + durationInMillis);

    const user = req.user; // Access the current logged-in user

    // Set profile visibility to false and calculate when it should become visible again
    user.isVisible = false;
    user.visibilityEndTime = visibilityEndTime;

    await user.save();

    res.status(200).json({
      status: true,
      message: `Profile visibility is now hidden for ${lockForDuration} ${unit}.`,
      visibilityEndTime,
    });
  } catch (err) {
    console.error(err);
    res.status(500).json({
      status: false,
      error: "Failed to set profile visibility",
    });
  }
};

// Controller to check profile visibility status
exports.checkProfileVisibility = async (req, res) => {
  try {
    const user = req.user; // Access the current logged-in user
    // console.log(user);

    const status = user.isVisible ? "visible" : "hidden";
    res.status(200).json({
      status: true,
      message: `Your profile is currently ${status}`,
      visibilityEndTime: user.visibilityEndTime || "N/A",
    });
  } catch (err) {
    console.error(err);
    res.status(500).json({
      status: false,
      error: "Failed to check profile visibility status",
    });
  }
};

// Controller to unhide profile manually
exports.unhideProfile = async (req, res) => {
  try {
    const user = req.user; // Access the current logged-in user
    user.isVisible = true;
    user.visibilityEndTime = null; // Reset the visibility end time
    await user.save();
    res.status(200).json({ status: true, message: "Profile is now visible." });
  } catch (err) {
    console.error(err);
    res.status(500).json({ status: false, error: "Failed to unhide profile" });
  }
};
